@@ -3,6 +3,7 @@
 
 import logging
 from abc import ABCMeta, abstractmethod
+from contextlib import ExitStack
 
 import torch
 import torch.nn as nn
@@ -16,8 +17,15 @@ from pyro.infer.predictive import _guess_max_plate_nesting
 from pyro.nn.module import PyroModule
 from pyro.optim import DCTAdam
 
-from .util import (MarkDCTParamMessenger, PrefixConditionMessenger, PrefixReplayMessenger, PrefixWarmStartMessenger,
-                   reshape_batch)
+from .util import (
+    MarkDCTParamMessenger,
+    PrefixConditionMessenger,
+    PrefixReplayMessenger,
+    PrefixWarmStartMessenger,
+    reshape_batch,
+    time_reparam_dct,
+    time_reparam_haar,
+)
 
 logger = logging.getLogger(__name__)
 
@@ -32,6 +40,7 @@ class ForecastingModel(PyroModule, metaclass=_ForecastingModelMeta):
 
     Derived classes must implement the :meth:`model` method.
     """
+
     def __init__(self):
         super().__init__()
         self._prefix_condition_data = {}
@@ -100,12 +109,14 @@ class ForecastingModel(PyroModule, metaclass=_ForecastingModelMeta):
         if noise_dist.event_dim == 0:
             if noise_dist.batch_shape[-2:] != prediction.shape[-2:]:
                 noise_dist = noise_dist.expand(
-                    noise_dist.batch_shape[:-2] + prediction.shape[-2:])
+                    noise_dist.batch_shape[:-2] + prediction.shape[-2:]
+                )
             noise_dist = noise_dist.to_event(2)
         elif noise_dist.event_dim == 1:
             if noise_dist.batch_shape[-1:] != prediction.shape[-2:-1]:
                 noise_dist = noise_dist.expand(
-                    noise_dist.batch_shape[:-1] + prediction.shape[-2:-1])
+                    noise_dist.batch_shape[:-1] + prediction.shape[-2:-1]
+                )
             noise_dist = noise_dist.to_event(1)
         assert noise_dist.event_dim == 2
         assert noise_dist.event_shape == prediction.shape[-2:]
@@ -146,7 +157,7 @@ class ForecastingModel(PyroModule, metaclass=_ForecastingModelMeta):
             noise = pyro.sample("residual", noise_dist)
             del self._prefix_condition_data["residual"]
 
-            assert noise.shape[-data.dim():] == right_pred.shape[-data.dim():]
+            assert noise.shape[-data.dim() :] == right_pred.shape[-data.dim() :]
             self._forecast = right_pred + noise
 
         # Move the "time" batch dim back to its original place.
@@ -167,7 +178,8 @@ class ForecastingModel(PyroModule, metaclass=_ForecastingModelMeta):
                 zero_data = data.new_zeros(()).expand(data.shape)
             else:  # forecasting
                 zero_data = data.new_zeros(()).expand(
-                    data.shape[:-2] + covariates.shape[-2:-1] + data.shape[-1:])
+                    data.shape[:-2] + covariates.shape[-2:-1] + data.shape[-1:]
+                )
             self._forecast = None
 
             self.model(zero_data, covariates)
@@ -224,6 +236,9 @@ class Forecaster(nn.Module):
         over all ``num_steps``, not the per-step decay factor.
     :param float clip_norm: Norm used for gradient clipping during
         optimization. Defaults to 10.0.
+    :param str time_reparam: If not None (default), reparameterize all
+        time-dependent variables via the Haar wavelet transform (if "haar") or
+        the discrete cosine transform (if "dct").
     :param bool dct_gradients: Whether to discrete cosine transform gradients
         in :class:`~pyro.optim.optim.DCTAdam`. Defaults to False.
     :param bool subsample_aware: whether to update gradient statistics only
@@ -241,29 +256,47 @@ class Forecaster(nn.Module):
         when publishing metrics.
     :param int log_every: Number of training steps between logging messages.
     """
-    def __init__(self, model, data, covariates, *,
-                 guide=None,
-                 init_loc_fn=init_to_sample,
-                 init_scale=0.1,
-                 create_plates=None,
-                 optim=None,
-                 learning_rate=0.01,
-                 betas=(0.9, 0.99),
-                 learning_rate_decay=0.1,
-                 clip_norm=10.0,
-                 dct_gradients=False,
-                 subsample_aware=False,
-                 num_steps=1001,
-                 num_particles=1,
-                 vectorize_particles=True,
-                 warm_start=False,
-                 log_every=100):
+
+    def __init__(
+        self,
+        model,
+        data,
+        covariates,
+        *,
+        guide=None,
+        init_loc_fn=init_to_sample,
+        init_scale=0.1,
+        create_plates=None,
+        optim=None,
+        learning_rate=0.01,
+        betas=(0.9, 0.99),
+        learning_rate_decay=0.1,
+        clip_norm=10.0,
+        time_reparam=None,
+        dct_gradients=False,
+        subsample_aware=False,
+        num_steps=1001,
+        num_particles=1,
+        vectorize_particles=True,
+        warm_start=False,
+        log_every=100
+    ):
         assert data.size(-2) == covariates.size(-2)
         super().__init__()
         self.model = model
+        if time_reparam == "haar":
+            model = poutine.reparam(model, time_reparam_haar)
+        elif time_reparam == "dct":
+            model = poutine.reparam(model, time_reparam_dct)
+        elif time_reparam is not None:
+            raise ValueError("unknown time_reparam: {}".format(time_reparam))
         if guide is None:
-            guide = AutoNormal(self.model, init_loc_fn=init_loc_fn, init_scale=init_scale,
-                               create_plates=create_plates)
+            guide = AutoNormal(
+                model,
+                init_loc_fn=init_loc_fn,
+                init_scale=init_scale,
+                create_plates=create_plates,
+            )
         self.guide = guide
 
         # Initialize.
@@ -273,19 +306,25 @@ class Forecaster(nn.Module):
         if dct_gradients:
             model = MarkDCTParamMessenger("time")(model)
             guide = MarkDCTParamMessenger("time")(guide)
-        elbo = Trace_ELBO(num_particles=num_particles,
-                          vectorize_particles=vectorize_particles)
+        elbo = Trace_ELBO(
+            num_particles=num_particles, vectorize_particles=vectorize_particles
+        )
         elbo._guess_max_plate_nesting(model, guide, (data, covariates), {})
         elbo.max_plate_nesting = max(elbo.max_plate_nesting, 1)  # force a time plate
 
         losses = []
         if num_steps:
             if optim is None:
-                optim = DCTAdam({"lr": learning_rate, "betas": betas,
-                                 "lrd": learning_rate_decay ** (1 / num_steps),
-                                 "clip_norm": clip_norm,
-                                 "subsample_aware": subsample_aware})
-            svi = SVI(self.model, self.guide, optim, elbo)
+                optim = DCTAdam(
+                    {
+                        "lr": learning_rate,
+                        "betas": betas,
+                        "lrd": learning_rate_decay ** (1 / num_steps),
+                        "clip_norm": clip_norm,
+                        "subsample_aware": subsample_aware,
+                    }
+                )
+            svi = SVI(model, guide, optim, elbo)
             for step in range(num_steps):
                 loss = svi.step(data, covariates) / data.numel()
                 if log_every and step % log_every == 0:
@@ -322,7 +361,7 @@ class Forecaster(nn.Module):
         return super().__call__(data, covariates, num_samples, batch_size)
 
     def forward(self, data, covariates, num_samples, batch_size=None):
-        assert data.size(-2) < covariates.size(-2)
+        assert data.size(-2) <= covariates.size(-2)
         assert isinstance(num_samples, int) and num_samples > 0
         if batch_size is not None:
             batches = []
@@ -336,13 +375,19 @@ class Forecaster(nn.Module):
         dim = -1 - self.max_plate_nesting
 
         with torch.no_grad():
-            with poutine.trace() as tr:
+            with poutine.block(), poutine.trace() as tr:
                 with pyro.plate("particles", num_samples, dim=dim):
                     self.guide(data, covariates)
-            with PrefixReplayMessenger(tr.trace):
-                with PrefixConditionMessenger(self.model._prefix_condition_data):
-                    with pyro.plate("particles", num_samples, dim=dim):
-                        return self.model(data, covariates)
+            with ExitStack() as stack:
+                if data.size(-2) < covariates.size(-2):
+                    stack.enter_context(PrefixReplayMessenger(tr.trace))
+                    stack.enter_context(
+                        PrefixConditionMessenger(self.model._prefix_condition_data)
+                    )
+                else:
+                    stack.enter_context(poutine.replay(trace=tr.trace))
+                with pyro.plate("particles", num_samples, dim=dim):
+                    return self.model(data, covariates)
 
 
 class HMCForecaster(nn.Module):
@@ -361,12 +406,14 @@ class HMCForecaster(nn.Module):
         For models not using covariates, pass a shaped empty tensor
         ``torch.empty(duration, 0)``.
     :type covariates: ~torch.Tensor
-
     :param int num_warmup: number of MCMC warmup steps.
     :param int num_samples: number of MCMC samples.
     :param int num_chains: number of parallel MCMC chains.
     :param bool dense_mass: a flag to control whether the mass matrix is dense
         or diagonal. Defaults to False.
+    :param str time_reparam: If not None (default), reparameterize all
+        time-dependent variables via the Haar wavelet transform (if "haar") or
+        the discrete cosine transform (if "dct").
     :param bool jit_compile: whether to use the PyTorch JIT to trace the log
         density computation, and use this optimized executable trace in the
         integrator. Defaults to False.
@@ -374,21 +421,52 @@ class HMCForecaster(nn.Module):
         doubling scheme of the :class:`~pyro.infer.mcmc.nuts.NUTS` sampler.
         Defaults to 10.
     """
-    def __init__(self, model, data, covariates=None, *,
-                 num_warmup=1000, num_samples=1000, num_chains=1,
-                 dense_mass=False, jit_compile=False, max_tree_depth=10):
+
+    def __init__(
+        self,
+        model,
+        data,
+        covariates=None,
+        *,
+        num_warmup=1000,
+        num_samples=1000,
+        num_chains=1,
+        time_reparam=None,
+        dense_mass=False,
+        jit_compile=False,
+        max_tree_depth=10
+    ):
         assert data.size(-2) == covariates.size(-2)
         super().__init__()
+        if time_reparam == "haar":
+            model = poutine.reparam(model, time_reparam_haar)
+        elif time_reparam == "dct":
+            model = poutine.reparam(model, time_reparam_dct)
+        elif time_reparam is not None:
+            raise ValueError("unknown time_reparam: {}".format(time_reparam))
         self.model = model
         max_plate_nesting = _guess_max_plate_nesting(model, (data, covariates), {})
         self.max_plate_nesting = max(max_plate_nesting, 1)  # force a time plate
 
-        kernel = NUTS(model, full_mass=dense_mass, jit_compile=jit_compile, ignore_jit_warnings=True,
-                      max_tree_depth=max_tree_depth, max_plate_nesting=max_plate_nesting)
-        mcmc = MCMC(kernel, warmup_steps=num_warmup, num_samples=num_samples, num_chains=num_chains)
+        kernel = NUTS(
+            model,
+            full_mass=dense_mass,
+            jit_compile=jit_compile,
+            ignore_jit_warnings=True,
+            max_tree_depth=max_tree_depth,
+            max_plate_nesting=max_plate_nesting,
+        )
+        mcmc = MCMC(
+            kernel,
+            warmup_steps=num_warmup,
+            num_samples=num_samples,
+            num_chains=num_chains,
+        )
         mcmc.run(data, covariates)
         # conditions to compute rhat
-        if (num_chains == 1 and num_samples >= 4) or (num_chains > 1 and num_samples >= 2):
+        if (num_chains == 1 and num_samples >= 4) or (
+            num_chains > 1 and num_samples >= 2
+        ):
             mcmc.summary()
 
         # inspect the model with particles plate = 1, so that we can reshape samples to
@@ -430,7 +508,7 @@ class HMCForecaster(nn.Module):
         return super().__call__(data, covariates, num_samples, batch_size)
 
     def forward(self, data, covariates, num_samples, batch_size=None):
-        assert data.size(-2) < covariates.size(-2)
+        assert data.size(-2) <= covariates.size(-2)
         assert isinstance(num_samples, int) and num_samples > 0
         if batch_size is not None:
             batches = []
@@ -445,13 +523,22 @@ class HMCForecaster(nn.Module):
 
         with torch.no_grad():
             weights = torch.ones(self._num_samples, device=data.device)
-            indices = torch.multinomial(weights, num_samples, replacement=num_samples > self._num_samples)
+            indices = torch.multinomial(
+                weights, num_samples, replacement=num_samples > self._num_samples
+            )
             for name, node in list(self._trace.nodes.items()):
                 sample = self._samples[name].index_select(0, indices)
-                node['value'] = sample.reshape(
-                    (num_samples,) + (1,) * (node['value'].dim() - sample.dim()) + sample.shape[1:])
+                node["value"] = sample.reshape(
+                    (num_samples,)
+                    + (1,) * (node["value"].dim() - sample.dim())
+                    + sample.shape[1:]
+                )
 
-            with PrefixReplayMessenger(self._trace):
-                with PrefixConditionMessenger(self.model._prefix_condition_data):
-                    with pyro.plate("particles", num_samples, dim=dim):
-                        return self.model(data, covariates)
+            with ExitStack() as stack:
+                if data.size(-2) < covariates.size(-2):
+                    stack.enter_context(PrefixReplayMessenger(self._trace))
+                    stack.enter_context(
+                        PrefixConditionMessenger(self.model._prefix_condition_data)
+                    )
+                with pyro.plate("particles", num_samples, dim=dim):
+                    return self.model(data, covariates)
