@@ -6,7 +6,11 @@ The :mod:`pyro.contrib.timeseries` module provides a collection of Bayesian time
 models useful for forecasting applications.
 """
 from pyro.contrib.timeseries.base import TimeSeriesModel
-from pyro.contrib.timeseries.gp import IndependentMaternGP, LinearlyCoupledMaternGP, DependentMaternGP
+from pyro.contrib.timeseries.gp import (
+    DependentMaternGP,
+    IndependentMaternGP,
+    LinearlyCoupledMaternGP,
+)
 from pyro.contrib.timeseries.lgssm import GenericLGSSM
 from pyro.contrib.timeseries.lgssmgp import GenericLGSSMWithGPNoiseModel
 
