@@ -12,7 +12,7 @@ from pyro.util import warn_if_nan
 
 
 def _product(factors):
-    result = 1.
+    result = 1.0
     for factor in factors:
         result = result * factor
     return result
@@ -52,6 +52,7 @@ class MarginalAssignment:
         final element denotes spurious detection, and
         ``.batch_shape == (num_frames, num_detections)``.
     """
+
     def __init__(self, exists_logits, assign_logits, bp_iters=None):
         assert exists_logits.dim() == 1, exists_logits.shape
         assert assign_logits.dim() == 2, assign_logits.shape
@@ -66,7 +67,9 @@ class MarginalAssignment:
         if bp_iters is None:
             exists, assign = compute_marginals(exists_logits, assign_logits)
         else:
-            exists, assign = compute_marginals_bp(exists_logits, assign_logits, bp_iters)
+            exists, assign = compute_marginals_bp(
+                exists_logits, assign_logits, bp_iters
+            )
 
         # Wrap the results in Distribution objects.
         # This adds a final logit=0 element denoting spurious detection.
@@ -101,7 +104,10 @@ class MarginalAssignmentSparse:
         final element denotes spurious detection, and
         ``.batch_shape == (num_frames, num_detections)``.
     """
-    def __init__(self, num_objects, num_detections, edges, exists_logits, assign_logits, bp_iters):
+
+    def __init__(
+        self, num_objects, num_detections, edges, exists_logits, assign_logits, bp_iters
+    ):
         assert edges.dim() == 2, edges.shape
         assert edges.shape[0] == 2, edges.shape
         assert exists_logits.shape == (num_objects,), exists_logits.shape
@@ -116,12 +122,17 @@ class MarginalAssignmentSparse:
 
         # This does all the work.
         exists, assign = compute_marginals_sparse_bp(
-            num_objects, num_detections, edges, exists_logits, assign_logits, bp_iters)
+            num_objects, num_detections, edges, exists_logits, assign_logits, bp_iters
+        )
 
         # Wrap the results in Distribution objects.
         # This adds a final logit=0 element denoting spurious detection.
-        padded_assign = torch.full((num_detections, num_objects + 1), -float('inf'),
-                                   dtype=assign.dtype, device=assign.device)
+        padded_assign = torch.full(
+            (num_detections, num_objects + 1),
+            -float("inf"),
+            dtype=assign.dtype,
+            device=assign.device,
+        )
         padded_assign[:, -1] = 0
         padded_assign[edges[0], edges[1]] = assign
         self.assign_dist = dist.Categorical(logits=padded_assign)
@@ -165,6 +176,7 @@ class MarginalAssignmentPersistent:
         final element denotes spurious detection, and
         ``.batch_shape == (num_frames, num_detections)``.
     """
+
     def __init__(self, exists_logits, assign_logits, bp_iters=None, bp_momentum=0.5):
         assert exists_logits.dim() == 1, exists_logits.shape
         assert assign_logits.dim() == 3, assign_logits.shape
@@ -180,7 +192,8 @@ class MarginalAssignmentPersistent:
             exists, assign = compute_marginals_persistent(exists_logits, assign_logits)
         else:
             exists, assign = compute_marginals_persistent_bp(
-                exists_logits, assign_logits, bp_iters, bp_momentum)
+                exists_logits, assign_logits, bp_iters, bp_momentum
+            )
 
         # Wrap the results in Distribution objects.
         # This adds a final logit=0 element denoting spurious detection.
@@ -203,11 +216,19 @@ def compute_marginals(exists_logits, assign_logits):
     dtype = exists_logits.dtype
     device = exists_logits.device
 
-    exists_probs = torch.zeros(2, num_objects, dtype=dtype, device=device)  # [not exist, exist]
-    assign_probs = torch.zeros(num_detections, num_objects + 1, dtype=dtype, device=device)
+    exists_probs = torch.zeros(
+        2, num_objects, dtype=dtype, device=device
+    )  # [not exist, exist]
+    assign_probs = torch.zeros(
+        num_detections, num_objects + 1, dtype=dtype, device=device
+    )
     for assign in itertools.product(range(num_objects + 1), repeat=num_detections):
-        assign_part = sum(assign_logits[j, i] for j, i in enumerate(assign) if i < num_objects)
-        for exists in itertools.product(*[[1] if i in assign else [0, 1] for i in range(num_objects)]):
+        assign_part = sum(
+            assign_logits[j, i] for j, i in enumerate(assign) if i < num_objects
+        )
+        for exists in itertools.product(
+            *[[1] if i in assign else [0, 1] for i in range(num_objects)]
+        ):
             exists_part = sum(exists_logits[i] for i, e in enumerate(exists) if e)
             prob = _exp(exists_part + assign_part)
             for i, e in enumerate(exists):
@@ -220,8 +241,8 @@ def compute_marginals(exists_logits, assign_logits):
     assign = assign_probs.log()
     exists = exists[1] - exists[0]
     assign = assign[:, :-1] - assign[:, -1:]
-    warn_if_nan(exists, 'exists')
-    warn_if_nan(assign, 'assign')
+    warn_if_nan(exists, "exists")
+    warn_if_nan(assign, "assign")
     return exists, assign
 
 
@@ -240,22 +261,29 @@ def compute_marginals_bp(exists_logits, assign_logits, bp_iters):
     message_e_to_a = torch.zeros_like(assign_logits)
     message_a_to_e = torch.zeros_like(assign_logits)
     for i in range(bp_iters):
-        message_e_to_a = -(message_a_to_e - message_a_to_e.sum(0, True) - exists_logits).exp().log1p()
+        message_e_to_a = (
+            -(message_a_to_e - message_a_to_e.sum(0, True) - exists_logits)
+            .exp()
+            .log1p()
+        )
         joint = (assign_logits + message_e_to_a).exp()
-        message_a_to_e = (assign_logits - torch.log1p(joint.sum(1, True) - joint)).exp().log1p()
-        warn_if_nan(message_e_to_a, 'message_e_to_a iter {}'.format(i))
-        warn_if_nan(message_a_to_e, 'message_a_to_e iter {}'.format(i))
+        message_a_to_e = (
+            (assign_logits - torch.log1p(joint.sum(1, True) - joint)).exp().log1p()
+        )
+        warn_if_nan(message_e_to_a, "message_e_to_a iter {}".format(i))
+        warn_if_nan(message_a_to_e, "message_a_to_e iter {}".format(i))
 
     # Convert from probs to logits.
     exists = exists_logits + message_a_to_e.sum(0)
     assign = assign_logits + message_e_to_a
-    warn_if_nan(exists, 'exists')
-    warn_if_nan(assign, 'assign')
+    warn_if_nan(exists, "exists")
+    warn_if_nan(assign, "assign")
     return exists, assign
 
 
-def compute_marginals_sparse_bp(num_objects, num_detections, edges,
-                                exists_logits, assign_logits, bp_iters):
+def compute_marginals_sparse_bp(
+    num_objects, num_detections, edges, exists_logits, assign_logits, bp_iters
+):
     """
     This implements approximate inference of pairwise marginals via
     loopy belief propagation, adapting the approach of [1].
@@ -271,8 +299,9 @@ def compute_marginals_sparse_bp(num_objects, num_detections, edges,
 
     def sparse_sum(x, dim, keepdim=False):
         assert dim in (0, 1)
-        x = (torch.zeros([num_objects, num_detections][dim], dtype=x.dtype, device=x.device)
-                  .scatter_add_(0, edges[1 - dim], x))
+        x = torch.zeros(
+            [num_objects, num_detections][dim], dtype=x.dtype, device=x.device
+        ).scatter_add_(0, edges[1 - dim], x)
         if keepdim:
             x = x[edges[1 - dim]]
         return x
@@ -280,17 +309,25 @@ def compute_marginals_sparse_bp(num_objects, num_detections, edges,
     message_e_to_a = torch.zeros_like(assign_logits)
     message_a_to_e = torch.zeros_like(assign_logits)
     for i in range(bp_iters):
-        message_e_to_a = -(message_a_to_e - sparse_sum(message_a_to_e, 0, True) - exists_factor).exp().log1p()
+        message_e_to_a = (
+            -(message_a_to_e - sparse_sum(message_a_to_e, 0, True) - exists_factor)
+            .exp()
+            .log1p()
+        )
         joint = (assign_logits + message_e_to_a).exp()
-        message_a_to_e = (assign_logits - torch.log1p(sparse_sum(joint, 1, True) - joint)).exp().log1p()
-        warn_if_nan(message_e_to_a, 'message_e_to_a iter {}'.format(i))
-        warn_if_nan(message_a_to_e, 'message_a_to_e iter {}'.format(i))
+        message_a_to_e = (
+            (assign_logits - torch.log1p(sparse_sum(joint, 1, True) - joint))
+            .exp()
+            .log1p()
+        )
+        warn_if_nan(message_e_to_a, "message_e_to_a iter {}".format(i))
+        warn_if_nan(message_a_to_e, "message_a_to_e iter {}".format(i))
 
     # Convert from probs to logits.
     exists = exists_logits + sparse_sum(message_a_to_e, 0)
     assign = assign_logits + message_e_to_a
-    warn_if_nan(exists, 'exists')
-    warn_if_nan(assign, 'assign')
+    warn_if_nan(exists, "exists")
+    warn_if_nan(assign, "assign")
     return exists, assign
 
 
@@ -308,7 +345,9 @@ def compute_marginals_persistent(exists_logits, assign_logits):
 
     total = 0
     exists_probs = torch.zeros(num_objects, dtype=dtype, device=device)
-    assign_probs = torch.zeros(num_frames, num_detections, num_objects, dtype=dtype, device=device)
+    assign_probs = torch.zeros(
+        num_frames, num_detections, num_objects, dtype=dtype, device=device
+    )
     for exists in itertools.product([0, 1], repeat=num_objects):
         exists = [i for i, e in enumerate(exists) if e]
         exists_part = _exp(sum(exists_logits[i] for i in exists))
@@ -322,7 +361,9 @@ def compute_marginals_persistent(exists_logits, assign_logits):
                 for objects in itertools.combinations(exists, n):
                     for detections in itertools.permutations(range(num_detections), n):
                         assign = tuple(zip(objects, detections))
-                        assign_map[assign] = _exp(sum(assign_logits[t, j, i] for i, j in assign))
+                        assign_map[assign] = _exp(
+                            sum(assign_logits[t, j, i] for i, j in assign)
+                        )
             assign_parts.append(assign_map)
             assign_sums.append(sum(assign_map.values()))
 
@@ -331,7 +372,7 @@ def compute_marginals_persistent(exists_logits, assign_logits):
         for i in exists:
             exists_probs[i] += prob
         for t in range(num_frames):
-            other_part = exists_part * _product(assign_sums[:t] + assign_sums[t + 1:])
+            other_part = exists_part * _product(assign_sums[:t] + assign_sums[t + 1 :])
             for assign, assign_part in assign_parts[t].items():
                 prob = other_part * assign_part
                 for i, j in assign:
@@ -340,12 +381,14 @@ def compute_marginals_persistent(exists_logits, assign_logits):
     # Convert from probs to logits.
     exists = exists_probs.log() - (total - exists_probs).log()
     assign = assign_probs.log() - (total - assign_probs.sum(-1, True)).log()
-    warn_if_nan(exists, 'exists')
-    warn_if_nan(assign, 'assign')
+    warn_if_nan(exists, "exists")
+    warn_if_nan(assign, "assign")
     return exists, assign
 
 
-def compute_marginals_persistent_bp(exists_logits, assign_logits, bp_iters, bp_momentum=0.5):
+def compute_marginals_persistent_bp(
+    exists_logits, assign_logits, bp_iters, bp_momentum=0.5
+):
     """
     This implements approximate inference of pairwise marginals via
     loopy belief propagation, adapting the approach of [1], [2].
@@ -372,31 +415,44 @@ def compute_marginals_persistent_bp(exists_logits, assign_logits, bp_iters, bp_m
     num_frames, num_detections, num_objects = assign_logits.shape
     dtype = assign_logits.dtype
     device = assign_logits.device
-    message_b_to_a = torch.zeros(num_frames, num_detections, num_objects, dtype=dtype, device=device)
-    message_a_to_b = torch.zeros(num_frames, num_detections, num_objects, dtype=dtype, device=device)
+    message_b_to_a = torch.zeros(
+        num_frames, num_detections, num_objects, dtype=dtype, device=device
+    )
+    message_a_to_b = torch.zeros(
+        num_frames, num_detections, num_objects, dtype=dtype, device=device
+    )
     message_b_to_e = torch.zeros(num_frames, num_objects, dtype=dtype, device=device)
     message_e_to_b = torch.zeros(num_frames, num_objects, dtype=dtype, device=device)
 
     for i in range(bp_iters):
         odds_a = (assign_logits + message_b_to_a).exp()
-        message_a_to_b = (old * message_a_to_b +
-                          new * (assign_logits - (odds_a.sum(2, True) - odds_a).log1p()))
-        message_b_to_e = (old * message_b_to_e +
-                          new * message_a_to_b.exp().sum(1).log1p())
-        message_e_to_b = (old * message_e_to_b +
-                          new * (exists_logits + message_b_to_e.sum(0) - message_b_to_e))
+        message_a_to_b = old * message_a_to_b + new * (
+            assign_logits - (odds_a.sum(2, True) - odds_a).log1p()
+        )
+        message_b_to_e = (
+            old * message_b_to_e + new * message_a_to_b.exp().sum(1).log1p()
+        )
+        message_e_to_b = old * message_e_to_b + new * (
+            exists_logits + message_b_to_e.sum(0) - message_b_to_e
+        )
         odds_b = message_a_to_b.exp()
-        message_b_to_a = (old * message_b_to_a -
-                          new * ((-message_e_to_b).exp().unsqueeze(1) + (1 + odds_b.sum(1, True) - odds_b)).log())
+        message_b_to_a = (
+            old * message_b_to_a
+            - new
+            * (
+                (-message_e_to_b).exp().unsqueeze(1)
+                + (1 + odds_b.sum(1, True) - odds_b)
+            ).log()
+        )
 
-        warn_if_nan(message_a_to_b, 'message_a_to_b iter {}'.format(i))
-        warn_if_nan(message_b_to_e, 'message_b_to_e iter {}'.format(i))
-        warn_if_nan(message_e_to_b, 'message_e_to_b iter {}'.format(i))
-        warn_if_nan(message_b_to_a, 'message_b_to_a iter {}'.format(i))
+        warn_if_nan(message_a_to_b, "message_a_to_b iter {}".format(i))
+        warn_if_nan(message_b_to_e, "message_b_to_e iter {}".format(i))
+        warn_if_nan(message_e_to_b, "message_e_to_b iter {}".format(i))
+        warn_if_nan(message_b_to_a, "message_b_to_a iter {}".format(i))
 
     # Convert from probs to logits.
     exists = exists_logits + message_b_to_e.sum(0)
     assign = assign_logits + message_b_to_a
-    warn_if_nan(exists, 'exists')
-    warn_if_nan(assign, 'assign')
+    warn_if_nan(exists, "exists")
+    warn_if_nan(assign, "assign")
     return exists, assign
