@@ -3,8 +3,7 @@
 
 import logging
 
-
-default_format = '%(levelname)s \t %(message)s'
+default_format = "%(levelname)s \t %(message)s"
 log = logging.getLogger("pyro")
 log.setLevel(logging.INFO)
 
