@@ -3,7 +3,11 @@
 
 from __future__ import absolute_import, division, print_function
 
-from pyro.nn.auto_reg_nn import AutoRegressiveNN, ConditionalAutoRegressiveNN, MaskedLinear
+from pyro.nn.auto_reg_nn import (
+    AutoRegressiveNN,
+    ConditionalAutoRegressiveNN,
+    MaskedLinear,
+)
 from pyro.nn.dense_nn import ConditionalDenseNN, DenseNN
 from pyro.nn.module import PyroModule, PyroParam, PyroSample, pyro_method
 
