@@ -1,8 +1,8 @@
 # Copyright (c) 2017-2019 Uber Technologies, Inc.
 # SPDX-License-Identifier: Apache-2.0
 
-from collections import OrderedDict
 import sys
+from collections import OrderedDict
 
 import opt_einsum
 
@@ -71,8 +71,9 @@ class Trace:
     """
 
     def __init__(self, graph_type="flat"):
-        assert graph_type in ("flat", "dense"), \
-            "{} not a valid graph type".format(graph_type)
+        assert graph_type in ("flat", "dense"), "{} not a valid graph type".format(
+            graph_type
+        )
         self.graph_type = graph_type
         self.nodes = OrderedDict()
         self._succ = OrderedDict()
@@ -104,12 +105,16 @@ class Trace:
         """
         if site_name in self:
             site = self.nodes[site_name]
-            if site['type'] != kwargs['type']:
+            if site["type"] != kwargs["type"]:
                 # Cannot sample or observe after a param statement.
-                raise RuntimeError("{} is already in the trace as a {}".format(site_name, site['type']))
-            elif kwargs['type'] != "param":
+                raise RuntimeError(
+                    "{} is already in the trace as a {}".format(site_name, site["type"])
+                )
+            elif kwargs["type"] != "param":
                 # Cannot sample after a previous sample statement.
-                raise RuntimeError("Multiple {} sites named '{}'".format(kwargs['type'], site_name))
+                raise RuntimeError(
+                    "Multiple {} sites named '{}'".format(kwargs["type"], site_name)
+                )
 
         # XXX should copy in case site gets mutated, or dont bother?
         self.nodes[site_name] = kwargs
@@ -188,17 +193,26 @@ class Trace:
                     log_p = site["log_prob_sum"]
                 else:
                     try:
-                        log_p = site["fn"].log_prob(site["value"], *site["args"], **site["kwargs"])
-                    except ValueError:
+                        log_p = site["fn"].log_prob(
+                            site["value"], *site["args"], **site["kwargs"]
+                        )
+                    except ValueError as e:
                         _, exc_value, traceback = sys.exc_info()
                         shapes = self.format_shapes(last_site=site["name"])
-                        raise ValueError("Error while computing log_prob_sum at site '{}':\n{}\n{}\n"
-                                         .format(name, exc_value, shapes)).with_traceback(traceback)
+                        raise ValueError(
+                            "Error while computing log_prob_sum at site '{}':\n{}\n{}\n".format(
+                                name, exc_value, shapes
+                            )
+                        ).with_traceback(traceback) from e
                     log_p = scale_and_mask(log_p, site["scale"], site["mask"]).sum()
                     site["log_prob_sum"] = log_p
                     if is_validation_enabled():
                         warn_if_nan(log_p, "log_prob_sum at site '{}'".format(name))
-                        warn_if_inf(log_p, "log_prob_sum at site '{}'".format(name), allow_neginf=True)
+                        warn_if_inf(
+                            log_p,
+                            "log_prob_sum at site '{}'".format(name),
+                            allow_neginf=True,
+                        )
                 result = result + log_p
         return result
 
@@ -213,20 +227,31 @@ class Trace:
             if site["type"] == "sample" and site_filter(name, site):
                 if "log_prob" not in site:
                     try:
-                        log_p = site["fn"].log_prob(site["value"], *site["args"], **site["kwargs"])
-                    except ValueError:
+                        log_p = site["fn"].log_prob(
+                            site["value"], *site["args"], **site["kwargs"]
+                        )
+                    except ValueError as e:
                         _, exc_value, traceback = sys.exc_info()
                         shapes = self.format_shapes(last_site=site["name"])
-                        raise ValueError("Error while computing log_prob at site '{}':\n{}\n{}"
-                                         .format(name, exc_value, shapes)).with_traceback(traceback)
+                        raise ValueError(
+                            "Error while computing log_prob at site '{}':\n{}\n{}".format(
+                                name, exc_value, shapes
+                            )
+                        ).with_traceback(traceback) from e
                     site["unscaled_log_prob"] = log_p
                     log_p = scale_and_mask(log_p, site["scale"], site["mask"])
                     site["log_prob"] = log_p
                     site["log_prob_sum"] = log_p.sum()
                     if is_validation_enabled():
-                        warn_if_nan(site["log_prob_sum"], "log_prob_sum at site '{}'".format(name))
-                        warn_if_inf(site["log_prob_sum"], "log_prob_sum at site '{}'".format(name),
-                                    allow_neginf=True)
+                        warn_if_nan(
+                            site["log_prob_sum"],
+                            "log_prob_sum at site '{}'".format(name),
+                        )
+                        warn_if_inf(
+                            site["log_prob_sum"],
+                            "log_prob_sum at site '{}'".format(name),
+                            allow_neginf=True,
+                        )
 
     def compute_score_parts(self):
         """
@@ -240,20 +265,31 @@ class Trace:
                 # Note that ScoreParts overloads the multiplication operator
                 # to correctly scale each of its three parts.
                 try:
-                    value = site["fn"].score_parts(site["value"], *site["args"], **site["kwargs"])
-                except ValueError:
+                    value = site["fn"].score_parts(
+                        site["value"], *site["args"], **site["kwargs"]
+                    )
+                except ValueError as e:
                     _, exc_value, traceback = sys.exc_info()
                     shapes = self.format_shapes(last_site=site["name"])
-                    raise ValueError("Error while computing score_parts at site '{}':\n{}\n{}"
-                                     .format(name, exc_value, shapes)).with_traceback(traceback)
+                    raise ValueError(
+                        "Error while computing score_parts at site '{}':\n{}\n{}".format(
+                            name, exc_value, shapes
+                        )
+                    ).with_traceback(traceback) from e
                 site["unscaled_log_prob"] = value.log_prob
                 value = value.scale_and_mask(site["scale"], site["mask"])
                 site["score_parts"] = value
                 site["log_prob"] = value.log_prob
                 site["log_prob_sum"] = value.log_prob.sum()
                 if is_validation_enabled():
-                    warn_if_nan(site["log_prob_sum"], "log_prob_sum at site '{}'".format(name))
-                    warn_if_inf(site["log_prob_sum"], "log_prob_sum at site '{}'".format(name), allow_neginf=True)
+                    warn_if_nan(
+                        site["log_prob_sum"], "log_prob_sum at site '{}'".format(name)
+                    )
+                    warn_if_inf(
+                        site["log_prob_sum"],
+                        "log_prob_sum at site '{}'".format(name),
+                        allow_neginf=True,
+                    )
 
     def detach_(self):
         """
@@ -268,26 +304,29 @@ class Trace:
         """
         :return: a list of names of observe sites
         """
-        return [name for name, node in self.nodes.items()
-                if node["type"] == "sample" and
-                node["is_observed"]]
+        return [
+            name
+            for name, node in self.nodes.items()
+            if node["type"] == "sample" and node["is_observed"]
+        ]
 
     @property
     def param_nodes(self):
         """
         :return: a list of names of param sites
         """
-        return [name for name, node in self.nodes.items()
-                if node["type"] == "param"]
+        return [name for name, node in self.nodes.items() if node["type"] == "param"]
 
     @property
     def stochastic_nodes(self):
         """
         :return: a list of names of sample sites
         """
-        return [name for name, node in self.nodes.items()
-                if node["type"] == "sample" and
-                not node["is_observed"]]
+        return [
+            name
+            for name, node in self.nodes.items()
+            if node["type"] == "sample" and not node["is_observed"]
+        ]
 
     @property
     def reparameterized_nodes(self):
@@ -295,10 +334,13 @@ class Trace:
         :return: a list of names of sample sites whose stochastic functions
             are reparameterizable primitive distributions
         """
-        return [name for name, node in self.nodes.items()
-                if node["type"] == "sample" and
-                not node["is_observed"] and
-                getattr(node["fn"], "has_rsample", False)]
+        return [
+            name
+            for name, node in self.nodes.items()
+            if node["type"] == "sample"
+            and not node["is_observed"]
+            and getattr(node["fn"], "has_rsample", False)
+        ]
 
     @property
     def nonreparam_stochastic_nodes(self):
@@ -369,19 +411,28 @@ class Trace:
                     log_prob = pack(log_prob, dim_to_symbol)
                     score_function = pack(score_function, dim_to_symbol)
                     entropy_term = pack(entropy_term, dim_to_symbol)
-                    packed["score_parts"] = ScoreParts(log_prob, score_function, entropy_term)
+                    packed["score_parts"] = ScoreParts(
+                        log_prob, score_function, entropy_term
+                    )
                     packed["log_prob"] = log_prob
-                    packed["unscaled_log_prob"] = pack(site["unscaled_log_prob"], dim_to_symbol)
+                    packed["unscaled_log_prob"] = pack(
+                        site["unscaled_log_prob"], dim_to_symbol
+                    )
                 elif "log_prob" in site:
                     packed["log_prob"] = pack(site["log_prob"], dim_to_symbol)
-                    packed["unscaled_log_prob"] = pack(site["unscaled_log_prob"], dim_to_symbol)
-            except ValueError:
+                    packed["unscaled_log_prob"] = pack(
+                        site["unscaled_log_prob"], dim_to_symbol
+                    )
+            except ValueError as e:
                 _, exc_value, traceback = sys.exc_info()
                 shapes = self.format_shapes(last_site=site["name"])
-                raise ValueError("Error while packing tensors at site '{}':\n  {}\n{}"
-                                 .format(site["name"], exc_value, shapes)).with_traceback(traceback)
+                raise ValueError(
+                    "Error while packing tensors at site '{}':\n  {}\n{}".format(
+                        site["name"], exc_value, shapes
+                    )
+                ).with_traceback(traceback) from e
 
-    def format_shapes(self, title='Trace Shapes:', last_site=None):
+    def format_shapes(self, title="Trace Shapes:", last_site=None):
         """
         Returns a string showing a table of the shapes of all sites in the
         trace.
@@ -390,34 +441,46 @@ class Trace:
             return title
         rows = [[title]]
 
-        rows.append(['Param Sites:'])
+        rows.append(["Param Sites:"])
         for name, site in self.nodes.items():
             if site["type"] == "param":
                 rows.append([name, None] + [str(size) for size in site["value"].shape])
             if name == last_site:
                 break
 
-        rows.append(['Sample Sites:'])
+        rows.append(["Sample Sites:"])
         for name, site in self.nodes.items():
             if site["type"] == "sample":
                 # param shape
                 batch_shape = getattr(site["fn"], "batch_shape", ())
                 event_shape = getattr(site["fn"], "event_shape", ())
-                rows.append([name + " dist", None] + [str(size) for size in batch_shape] +
-                            ["|", None] + [str(size) for size in event_shape])
+                rows.append(
+                    [name + " dist", None]
+                    + [str(size) for size in batch_shape]
+                    + ["|", None]
+                    + [str(size) for size in event_shape]
+                )
 
                 # value shape
                 event_dim = len(event_shape)
                 shape = getattr(site["value"], "shape", ())
-                batch_shape = shape[:len(shape) - event_dim]
-                event_shape = shape[len(shape) - event_dim:]
-                rows.append(["value", None] + [str(size) for size in batch_shape] +
-                            ["|", None] + [str(size) for size in event_shape])
+                batch_shape = shape[: len(shape) - event_dim]
+                event_shape = shape[len(shape) - event_dim :]
+                rows.append(
+                    ["value", None]
+                    + [str(size) for size in batch_shape]
+                    + ["|", None]
+                    + [str(size) for size in event_shape]
+                )
 
                 # log_prob shape
                 if "log_prob" in site:
                     batch_shape = getattr(site["log_prob"], "shape", ())
-                    rows.append(["log_prob", None] + [str(size) for size in batch_shape] + ["|", None])
+                    rows.append(
+                        ["log_prob", None]
+                        + [str(size) for size in batch_shape]
+                        + ["|", None]
+                    )
             if name == last_site:
                 break
 
@@ -450,10 +513,12 @@ def _format_table(rows):
                 j += 1
             else:
                 cols[j].append(cell)
-        cols = [[""] * (width - len(col)) + col
-                if direction == 'r' else
-                col + [""] * (width - len(col))
-                for width, col, direction in zip(column_widths, cols, 'rrl')]
+        cols = [
+            [""] * (width - len(col)) + col
+            if direction == "r"
+            else col + [""] * (width - len(col))
+            for width, col, direction in zip(column_widths, cols, "rrl")
+        ]
         rows[i] = sum(cols, [])
 
     # compute cell widths
@@ -463,6 +528,7 @@ def _format_table(rows):
             cell_widths[j] = max(cell_widths[j], len(cell))
 
     # justify cells
-    return "\n".join(" ".join(cell.rjust(width)
-                              for cell, width in zip(row, cell_widths))
-                     for row in rows)
+    return "\n".join(
+        " ".join(cell.rjust(width) for cell, width in zip(row, cell_widths))
+        for row in rows
+    )
